@@ -13,10 +13,6 @@
 //! A library to create HTTP response content for APIs based on
 //! [RFC7807](https://tools.ietf.org/html/rfc7807).
 //!
-//! This library depends on [serde](https://serde.rs/).
-//!
-//! The `HttpApiProblem` struct implements `Serialize` and `Deserialize`.
-//!
 //! ## Usage
 //!
 //! Add this to your `Cargo.toml`:
@@ -42,7 +38,7 @@
 //!     .set_instance("/on/1234/do/something");
 //!
 //! assert_eq!(Some("https://httpstatuses.com/404".to_string()), p.type_url);
-//! assert_eq!(Some(404), p.status);
+//! assert_eq!(Some(HttpStatusCode::NotFound), p.status);
 //! assert_eq!("Not Found".to_string(), p.title);
 //! assert_eq!(Some("detailed explanation".to_string()), p.detail);
 //! assert_eq!(Some("/on/1234/do/something".to_string()), p.instance);
@@ -59,7 +55,7 @@
 //!     .set_instance("/on/1234/do/something");
 //!
 //! assert_eq!(Some("https://httpstatuses.com/428".to_string()), p.type_url);
-//! assert_eq!(Some(428), p.status);
+//! assert_eq!(Some(HttpStatusCode::PreconditionRequired), p.status);
 //! assert_eq!("Precondition Required".to_string(), p.title);
 //! assert_eq!(Some("detailed explanation".to_string()), p.detail);
 //! assert_eq!(Some("/on/1234/do/something".to_string()), p.instance);
@@ -67,10 +63,25 @@
 //!
 //! ## Features
 //!
-//! To directly construct from `iron::status::StatusCode` from the [iron-framework](http://ironframework.io/)
-//! use `feature` `iron` which adds `From<iron::status::StatusCode> for HttpStatusCode`.
-//!
-//! Ther `serde` feature provides `Serialize` and `Deserialize` for `HttpStatusCode`.
+//! ### serde
+//! 
+//! The `serde` feature provides `Serialize` and `Deserialize` for `HttpApiProblem`.
+//! 
+//! ### serde + serde_json
+//! 
+//! `HttpApiProblem` provides a method to `` wich is a `Vec[u8]`
+//! 
+//! ### iron
+//! 
+//! There is a conversion between `iron`s StatusCode and `HttpStatusCode` back and forth.
+//! 
+//! ### iron + serde + serde_json 
+//! 
+//! The `HttpApiProblem` provides a method `to_iron_response` which constructs an iron `Response`.
+//! If the `status` field of the `HttpApiProblem` is `None` `500 - Internal Server Error` is the default.
+//! 
+//! `From<HttpApiProblem` for `iron::response::Response` will also be there. It simply calls
+//! `to_iron_response`.
 //!
 //! ## License
 //!
@@ -83,11 +94,18 @@
 #[macro_use]
 extern crate serde;
 
+#[cfg(all(feature = "serde", feature = "serde_json"))]
+#[macro_use]
+extern crate serde_json;
+
 #[cfg(feature = "iron")]
 extern crate iron;
 
 use std::fmt;
 
+#[cfg(feature = "serde")]
+use serde::{Serialize, Deserialize, Serializer, Deserializer};
+
 /// The recommended media type when serialized to JSON
 pub static PROBLEM_JSON_MEDIA_TYPE: &'static str = "application/problem+json";
 
@@ -122,7 +140,7 @@ pub struct HttpApiProblem {
     /// The HTTP status code [RFC7231, Section 6](https://tools.ietf.org/html/rfc7231#section-6)
     /// generated by the origin server for this occurrence of the problem.
     #[cfg_attr(feature = "serde", serde(skip_serializing_if = "Option::is_none"))]
-    pub status: Option<u16>,
+    pub status: Option<HttpStatusCode>,
     /// A short, human-readable summary of the problem
     /// type. It SHOULD NOT change from occurrence to occurrence of the
     /// problem, except for purposes of localization (e.g., using
@@ -178,7 +196,7 @@ impl HttpApiProblem {
     /// let p = HttpApiProblem::with_title_and_type_from_status(503);
     ///
     /// assert_eq!(Some("https://httpstatuses.com/503".to_string()), p.type_url);
-    /// assert_eq!(Some(503), p.status);
+    /// assert_eq!(Some(HttpStatusCode::ServiceUnavailable), p.status);
     /// assert_eq!("Service Unavailable", &p.title);
     /// assert_eq!(None, p.detail);
     /// assert_eq!(None, p.instance);
@@ -187,7 +205,7 @@ impl HttpApiProblem {
         let status = status.into();
         HttpApiProblem {
             type_url: Some(format!("https://httpstatuses.com/{}", status.to_u16())),
-            status: Some(status.to_u16()),
+            status: Some(status.into()),
             title: status.title().to_string(),
             detail: None,
             instance: None,
@@ -204,7 +222,7 @@ impl HttpApiProblem {
     /// let p = HttpApiProblem::with_title_from_status(404);
     ///
     /// assert_eq!(None, p.type_url);
-    /// assert_eq!(Some(404), p.status);
+    /// assert_eq!(Some(HttpStatusCode::NotFound), p.status);
     /// assert_eq!("Not Found", p.title);
     /// assert_eq!(None, p.detail);
     /// assert_eq!(None, p.instance);
@@ -213,7 +231,7 @@ impl HttpApiProblem {
         let status = status.into();
         HttpApiProblem {
             type_url: None,
-            status: Some(status.to_u16()),
+            status: Some(status.into()),
             title: status.title().to_string(),
             detail: None,
             instance: None,
@@ -253,7 +271,7 @@ impl HttpApiProblem {
     /// let p = HttpApiProblem::new("Error").set_status(404);
     ///
     /// assert_eq!(None, p.type_url);
-    /// assert_eq!(Some(404), p.status);
+    /// assert_eq!(Some(HttpStatusCode::NotFound), p.status);
     /// assert_eq!("Error", p.title);
     /// assert_eq!(None, p.detail);
     /// assert_eq!(None, p.instance);
@@ -261,7 +279,7 @@ impl HttpApiProblem {
     pub fn set_status<T: Into<HttpStatusCode>>(self, status: T) -> HttpApiProblem {
         let status = status.into();
         let mut s = self;
-        s.status = Some(status.to_u16());
+        s.status = Some(status.into());
         s
     }
 
@@ -331,6 +349,34 @@ impl HttpApiProblem {
         s.instance = Some(instance.into());
         s
     }
+
+    #[cfg(all(feature = "serde", feature = "serde_json"))]
+    pub fn json_bytes(&self) -> Vec<u8> {
+        serde_json::to_vec(self).unwrap()
+    }
+
+    /// Creates an `Iron` response.
+    /// 
+    /// If status is `None` `500 - Internal Server Error` is the
+    /// default.
+    #[cfg(all(feature = "serde", feature = "serde_json", feature = "iron"))]
+    pub fn to_iron_response(self) -> ::iron::response::Response {
+        use iron::*;
+        use iron::headers::{Headers, ContentType};
+        use iron::status::Status;
+        use iron::mime::{Mime, TopLevel, SubLevel, Attr, Value};
+
+        let status: Status = self.status.unwrap_or(HttpStatusCode::InternalServerError).into();
+
+        let mut response = Response::with((status, self.json_bytes()));
+        let mime: Mime = PROBLEM_JSON_MEDIA_TYPE.parse().unwrap();
+        response.headers.set(
+            ContentType(mime));
+
+
+        response
+
+    }
 }
 
 impl From<HttpStatusCode> for HttpApiProblem {
@@ -339,6 +385,13 @@ impl From<HttpStatusCode> for HttpApiProblem {
     }
 }
 
+#[cfg(all(feature = "serde", feature = "serde_json", feature = "iron", test))]
+impl From<HttpApiProblem> for ::iron::response::Response {
+    fn from(p: HttpApiProblem) -> ::iron::response::Response {
+        p.to_iron_response()
+    }
+}
+
 /// An HTTP status code (`status-code` in RFC 7230 et al.).
 ///
 /// This enum contains all common status codes and an Unregistered
@@ -656,6 +709,24 @@ impl fmt::Display for HttpStatusCode {
     }
 }
 
+#[cfg(feature = "serde")]
+impl Serialize for HttpStatusCode {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+        where S: Serializer
+    {
+        serializer.serialize_u16(self.to_u16())
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de> Deserialize<'de> for HttpStatusCode {
+    fn deserialize<D>(deserializer: D) -> Result<HttpStatusCode, D::Error>
+        where D: Deserializer<'de>
+    {
+        u16::deserialize(deserializer).map(Into::into)
+    }
+}
+
 #[cfg(feature = "iron")]
 impl From<::iron::status::Status> for HttpStatusCode {
     fn from(iron_status: ::iron::status::Status) -> HttpStatusCode {
